# Generated by roxygen2: do not edit by hand

S3method(animate,default)
S3method(animate,gganim)
S3method(c,TransmuteFactory)
S3method(ggplot_add,EaseAes)
S3method(ggplot_add,EnterFactory)
S3method(ggplot_add,ExitFactory)
S3method(ggplot_add,Shadow)
S3method(ggplot_add,Transition)
S3method(ggplot_add,View)
S3method(ggplot_build,gganim)
S3method(group_column,default)
S3method(layer_type,GeomPath)
S3method(layer_type,GeomPolygon)
S3method(layer_type,GeomRibbon)
S3method(layer_type,GeomSf)
S3method(layer_type,GeomSmooth)
S3method(layer_type,Stat)
S3method(layer_type,StatContour)
S3method(layer_type,default)
S3method(match_shapes,StatContour)
S3method(match_shapes,default)
S3method(plot,gganim)
S3method(print,gganim)
S3method(print,gif_image)
S3method(print,sprite_image)
S3method(print,video_file)
S3method(save_animation,"magick-image")
S3method(save_animation,default)
S3method(save_animation,gif_image)
S3method(save_animation,video_file)
S3method(split,gif_image)
S3method(split,sprite_image)
S3method(split,video_file)
S3method(tween_before_stat,default)
export(Shadow)
export(ShadowMark)
export(ShadowNull)
export(ShadowTrail)
export(ShadowWake)
export(Transition)
export(TransitionComponents)
export(TransitionEvents)
export(TransitionFilter)
export(TransitionLayers)
export(TransitionManual)
export(TransitionNull)
export(TransitionReveal)
export(TransitionStates)
export(TransitionTime)
export(View)
export(ViewFollow)
export(ViewStep)
export(ViewStepManual)
export(ViewZoom)
export(ViewZoomManual)
export(anim_save)
export(animate)
export(av_renderer)
export(ease_aes)
export(enter_appear)
export(enter_drift)
export(enter_fade)
export(enter_fly)
export(enter_grow)
export(enter_manual)
export(enter_recolor)
export(enter_recolour)
export(enter_reset)
export(exit_disappear)
export(exit_drift)
export(exit_fade)
export(exit_fly)
export(exit_manual)
export(exit_recolor)
export(exit_recolour)
export(exit_reset)
export(exit_shrink)
export(ffmpeg_renderer)
export(file_renderer)
export(frame_vars)
export(gg_animate)
export(gganimate)
export(gif_file)
export(gifski_renderer)
export(group_column)
export(knit_print.gganim)
export(knit_print.gif_image)
export(knit_print.sprite_image)
export(knit_print.video_file)
export(last_animation)
export(layer_type)
export(magick_renderer)
export(match_shapes)
export(save_animation)
export(set_last_animation)
export(shadow_mark)
export(shadow_null)
export(shadow_trail)
export(shadow_wake)
export(split_animation)
export(sprite_file)
export(sprite_renderer)
export(transition_components)
export(transition_events)
export(transition_filter)
export(transition_layers)
export(transition_manual)
export(transition_null)
export(transition_reveal)
export(transition_states)
export(transition_time)
export(tween_before_stat)
export(video_file)
export(view_follow)
export(view_static)
export(view_step)
export(view_step_manual)
export(view_zoom)
export(view_zoom_manual)
import(rlang)
importFrom(ggplot2,CoordCartesian)
importFrom(ggplot2,FacetNull)
importFrom(ggplot2,Layout)
importFrom(ggplot2,geom_blank)
importFrom(ggplot2,ggplot_add)
importFrom(ggplot2,ggplot_build)
importFrom(ggplot2,ggplot_gtable)
importFrom(ggplot2,ggproto)
importFrom(ggplot2,is_ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,waiver)
importFrom(glue,glue)
importFrom(glue,glue_data)
importFrom(grDevices,bmp)
importFrom(grDevices,col2rgb)
importFrom(grDevices,dev.off)
importFrom(grDevices,jpeg)
importFrom(grDevices,png)
importFrom(grDevices,svg)
importFrom(grDevices,tiff)
importFrom(grid,convertHeight)
importFrom(grid,convertWidth)
importFrom(grid,grid.draw)
importFrom(grid,grid.newpage)
importFrom(grid,pushViewport)
importFrom(grid,seekViewport)
importFrom(grid,upViewport)
importFrom(progress,progress_bar)
importFrom(rlang,as_function)
importFrom(rlang,enquo)
importFrom(rlang,eval_tidy)
importFrom(rlang,quo)
importFrom(rlang,quo_is_missing)
importFrom(rlang,quo_is_null)
importFrom(rlang,quo_name)
importFrom(rlang,quos)
importFrom(rlang,quos_auto_name)
importFrom(scales,alpha)
importFrom(stringi,stri_detect)
importFrom(stringi,stri_match)
importFrom(stringi,stri_replace)
importFrom(tweenr,keep_state)
importFrom(tweenr,tween_along)
importFrom(tweenr,tween_at)
importFrom(tweenr,tween_components)
importFrom(tweenr,tween_constant)
importFrom(tweenr,tween_events)
importFrom(tweenr,tween_numeric)
importFrom(tweenr,tween_state)
importFrom(utils,modifyList)
