#' @importFrom rlang %||%
#' @aliases gganimate-package
#'
#' @section Good starting points:
#' The [animate()] function documentation describes how to specify how the
#' animation is rendered, whereas the documentation for the different transition
#' functions, e.g. [transition_states()], describe how to declare the animation.
#'
#' Both [ease_aes()] and the different [enter and exit][enter_exit] functions
#' are good to understand in order to modify tweening of the animation.
#'
#' Apart from the included documentation there is also a few web ressources
#' worth mentioning:
#'
#' - The [gganimate wiki](https://github.com/thomasp85/gganimate/wiki) includes
#'   user supplied example walkthroughs
#' - The [learngganimate](https://github.com/ropensci-archive/learngganimate)
#'   project by rOpenSci Labs includes a long range of detailed explorations of
#'   the different gganimate functions
#'
'_PACKAGE'
