% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/animate.R
\name{gganimate}
\alias{gganimate}
\alias{gg_animate}
\title{Catch attempt to use the old API}
\usage{
gganimate(...)

gg_animate(...)
}
\description{
Catch attempt to use the old API
}
\keyword{internal}
